package practicas;

/**
 * Clase: Protagonista
 * Descripcin: Esta clase representa al protagonista en el juego del comecocos. 
 * Creada por: Manuel Pereira
 * Fecha: 13-feb-2007
 */

public class Protagonista {

    /** Variables propias del protagonista para la posicin y la direccin */
    protected int posicionX;
    protected int posicionY;
    protected int direccion;

    /** Constantes para las 4 direcciones posibles */
    protected static final int DIRECCION_ARRIBA = 0;
    protected static final int DIRECCION_ABAJO = 1;
    protected static final int DIRECCION_IZQUIERDA = 2;
    protected static final int DIRECCION_DERECHA = 3;

    /**
     * Constructor que recibe toda la informacin sobre el protagonista
     * @param posicionX Posicin X inicial del protagonista
     * @param posicionY Posicin Y inicial del protagonista
     * @param direccion Direccin inicial del protagonista 
     */
    public Protagonista(int posicionX, int posicionY, int direccion) {
        this.posicionX = posicionX;
        this.posicionY = posicionY;
        this.direccion = direccion;
    }

    /**
     * Mtodo que indica al protagonista que se mueva. Para ello se le pasan como
     * parmetro las casillas del tablero
     * @param casillas Las casillas del tablero
     * @param teclaPulsada Tecla pulsada por el usuario final
     */
    public void muevete(Casilla[][] casillas, String teclaPulsada) {

        // Modifica la direccin si se ha pulsado una de las teclas de cambio de direccin
        if(teclaPulsada.equals("W") || teclaPulsada.equals("w")) {
            direccion = DIRECCION_ARRIBA;
        }
        else if(teclaPulsada.equals("S") || teclaPulsada.equals("s")) {
            direccion = DIRECCION_ABAJO;
        }
        else if(teclaPulsada.equals("A") || teclaPulsada.equals("a")) {
            direccion = DIRECCION_IZQUIERDA;
        }
        else if(teclaPulsada.equals("D") || teclaPulsada.equals("d")) {
            direccion = DIRECCION_DERECHA;
        }
        // Si se ha pulsado la tecla Q, sale de la partida
        else if(teclaPulsada.equals("Q") || teclaPulsada.equals("q")) {
            System.exit(-1);
        }

        // Se mueve
        switch(direccion) {
            case DIRECCION_ARRIBA:
                if(casillas[posicionY-1][posicionX].esPisable()) {
                    posicionY = posicionY - 1;
                }
                break;
            case DIRECCION_ABAJO:
                if(casillas[posicionY+1][posicionX].esPisable()) {
                    posicionY = posicionY + 1;
                }
                break;
            case DIRECCION_IZQUIERDA:
                if(casillas[posicionY][posicionX-1].esPisable()) {
                    posicionX = posicionX - 1;
                }
                break;
            case DIRECCION_DERECHA:
                if(casillas[posicionY][posicionX+1].esPisable()) {
                    posicionX = posicionX + 1;
                }
                break;
        }
    }

    /**
     * Comprueba si la posicin del protagonista coincide con la de otro dado
     * @param otro Protagonista con el que comparar la posicin
     * @return true si coinciden, false si no
     */
    public boolean coincide(Protagonista otro) {
        return otro.getPosicionX() == posicionX && otro.getPosicionY() == posicionY;
    }

    /**
     * Comprueba si la posicin de este protagonista coincide con una dada
     * @param x Posicin x dada
     * @param y Posicin y dada
     * @return true si coinciden, false si no
     */
    public boolean coincide(int x, int y) {
        return x == posicionX && y == posicionY;
    }

    /**
     * Devuelve la posicin x del protagonista
     * @return La posicin x del protagonista
     */
    public int getPosicionX() {
        return posicionX;
    }

    /**
     * Devuelve la posicin y del protagonista
     * @return La posicin y del protagonista
     */
    public int getPosicionY() {
        return posicionY;
    }

    /**
     * Establece la posicin X del protagonista
     * @param posicionX La posicin X
     */
    public void setPosicionX(int posicionX) {
        this.posicionX = posicionX;
    }

    /**
     * Establece la posicin Y del protagonista
     * @param posicionY La posicin Y
     */
    public void setPosicionY(int posicionY) {
        this.posicionY = posicionY;
    }

    /**
     * Establece la direccin del protagonista
     * @param direccion La direccin
     */
    public void setDireccion(int direccion) {
        this.direccion = direccion;
    }

    /**
     * Devuelve la direccin actual del protagonista
     * @return La direccin actual del protagonista
     */
    public int getDireccion() {
        return direccion;
    }
}

