package practicas;

/**
 * Clase: Individuo
 * Descripcin: Esta clase representa un individuo en el juego del comecocos. Un individuo
 * es cualquier ente movible en el juego (tanto el protagonista como los enemigos)
 * Creada por: Manuel Pereira
 * Fecha: 13-feb-2007
 * Modificada por: Daniel Prez
 * Fecha: 10-may-2007
 */

public class Individuo {

    /** Variables propias del individuo para la posicin y la direccin */
    protected int posicionX;
    protected int posicionY;
    protected int direccion;

    /** Indican la posicin inicial del individuo al inicio del juego y de cada vida */
    protected int inicialX;
    protected int inicialY;

    /** Constantes para las 4 direcciones posibles */
    protected static final int DIRECCION_ARRIBA = 0;
    protected static final int DIRECCION_ABAJO = 1;
    protected static final int DIRECCION_IZQUIERDA = 2;
    protected static final int DIRECCION_DERECHA = 3;

    /**
     * Constructor para un individuo, indicando su posicin y direccin inicial
     * @param posicionX Posicin x inicial del individuo
     * @param posicionY Posicin y inicial del individuo
     * @param direccion Direccin inicial del individuo
     */
    public Individuo(int posicionX, int posicionY, int direccion) {
        this.posicionX = posicionX;
        this.inicialX =posicionX;
        this.posicionY = posicionY;
        this.inicialY = posicionY;
        this.direccion = direccion;
    }

    /**
     * Sita al individuo en su posicin inicial
     */
    public void situarInicio() {
        posicionX = inicialX;
        posicionY = inicialY;
    }
    
    /**
     * Comprueba si la posicin de este individuo coincide con la de otro dado
     * @param otro Individuo con el que comparar la posicin
     * @return true si coinciden, false si no
     */
    public boolean coincide(Individuo otro) {
        return otro.getPosicionX() == posicionX && otro.getPosicionY() == posicionY;
    }

    /**
     * Comprueba si la posicin de este individuo coincide con una dada
     * @param x Posicin x dada
     * @param y Posicin y dada
     * @return true si coinciden, false si no
     */
    public boolean coincide(int x, int y) {
        return x == posicionX && y == posicionY;
    }

    /**
     * Comprueba si la posicin de este individuo es adyacente a una dada
     * @param x Posicin x dada
     * @param y Posicin y dada
     * @return true si es adyacente, false si no
     */   
    public boolean esAdyacente(int x, int y) {
    	return coincide(x,y-1)||coincide(x,y+1)||coincide(x-1,y)||coincide(x+1,y);
    }

    /**
     * Devuelve la posicin x del individuo
     * @return La posicin x del inidividuo
     */
    public int getPosicionX() {
        return posicionX;
    }

    /**
     * Devuelve la posicin y del individuo
     * @return La posicin y del inidividuo
     */
    public int getPosicionY() {
        return posicionY;
    }

    /**
     * Devuelve la direccin actual del individuo
     * @return La direccin actual del individuo
     */
    public int getDireccion() {
        return direccion;
    }

    /**
     * Mueve al individuo en la direccin actual, si es posible
     * @param casillas Casillas del tablero
     */
    public void muevete(Casilla[][] casillas) {
        switch(direccion) {
            case DIRECCION_ARRIBA:
                if(casillas[posicionY-1][posicionX].esPisable()) {
                    posicionY = posicionY - 1;
                }
                break;
            case DIRECCION_ABAJO:
                if(casillas[posicionY+1][posicionX].esPisable()) {
                    posicionY = posicionY + 1;
                }
                break;
            case DIRECCION_IZQUIERDA:
                if(casillas[posicionY][posicionX-1].esPisable()) {
                    posicionX = posicionX - 1;
                }
                break;
            case DIRECCION_DERECHA:
                if(casillas[posicionY][posicionX+1].esPisable()) {
                    posicionX = posicionX + 1;
                }
                break;
        }
    }
}
