package practicas;

/**
 * Clase: Movimiento
 * Descripcin: Esta clase representa un posible movimiento de un enemigo del comecocos
 * Creada por: Daniel Prez
 * Fecha: 10-may-2007
 */

public class Movimiento {

	/** Variable que representa el valor estimado de realizar dicho movimiento */
	private double valor;
	
	/** Variables propias del movimiento para la posicin y la direccin */
    protected int posicionX;
    protected int posicionY;
    protected int direccion; 
	
    /**
     * Constructor de un movimiento de un enemigo
     * @param x         Posicion X final del movimiento
     * @param y         Posicion Y final del movimiento
     * @param direccion Direccin del movimiento de un enemigo
     * @param valor     Distancia estimada hasta el objetivo
     */
	public Movimiento(int x,int y,int direccion,int a,int b){
		posicionX=x;
		posicionY=y;
		this.direccion=direccion;
		valor=estimarDistancia(x,y,a,b);
	}
 
	 /**
     * Mtodo auxiliar que devuelve el valor absoluto de un nmero entero
     * @param a Nmero entero introducido
     * @return valor absoluto del nmero introducido 
     */   
    public int abs(int a){
		if (a<0) a=-a;
		return a;
	}

    /**
     * Mtodo que evala un movimiento estimando la distancia entre
     * dos posiciones
     * @param a     Posicin X de la primera posicin
     * @param b     Posicin Y de la primera posicin
     * @param x     Posicin X de la segunda posicin
     * @param y     Posicin Y de la segunda posicin
     * @return valor estimado 
     */ 
    public double estimarDistancia(int a, int b, int x, int y) {
    	return abs(a-x)+abs(b-y);
    }
	
	/**
	 * Asigna una posicion X a un movimiento
	 * @param x Posicion X
	 */
	public void setPosicionX(int x) {
	    this.posicionX = x;
	}
	
	/**
	 * Devuelve la posicion X final tras completar un movimiento
	 * @return la posicin X final del movimiento
	 */
	public int getPosicionX() {
	    return posicionX;
	}
	
	/**
	 * Asigna una posicion Y a un movimiento
	 * @param y Posicion Y
	 */
	public void setPosicionY(int y) {
	    this.posicionY = y;
	}
	
	/**
	 * Devuelve la posicion Y final tras completar un movimiento
	 * @return la posicin Y final del movimiento
	 */
	public int getPosicionY() {
	    return posicionY;
	}
	
	/**
	 * Asigna una direccin a un movimiento
	 * @param direccin Direccin del movimiento
	 */
	public void setDireccion(int direccion) {
	    this.direccion = direccion;
	}
	
	/**
	 * Devuelve la direccion de un movimiento
	 * @return la direccin del movimiento
	 */
	public int getDireccion() {
	    return direccion;
	}
	
	/**
	 * Asigna un valor a un movimiento
	 * @param valor El valor del movimiento
	 */
	public void setValor(double valor) {
	    this.valor = valor;
	}

	/**
	 * Devuelve el valor de un movimiento
	 * @return el valor del movimiento
	 */
	public double getValor() {
	    return valor;
	}

	/**
	 * Mtodo privado que comprueba si el valor actual de la casilla es menor que otro
	 * @param c Casilla
	 * @return true si el valor es menor que el de la Casilla c, false si no lo es
	 */
	public boolean menor(Movimiento m) {
	if (this.valor<m.getValor()) return true;
	else return false;
	}

	/**
	 * Mtodo privado que comprueba si el valor actual de la casilla es mayor que otro
	 * @param c Casilla
	 * @return true si el valor es mayor que el de la Casilla c, false si no lo es
	 */
	public boolean mayor(Movimiento m) {
	if (this.valor>m.getValor()) return true;
	else return false;
	}
	
	/**
	 * Mtodo que imprime el movimiento por pantalla
	 */
	public void mostrar() {
		System.out.println(direccion+"("+posicionX+","+posicionY+"): "+valor);
		}
}
