package practicas;

/**
 * Clase: Casilla
 * Descripcin: Esta clase representa una casilla del tablero del juego del comecocos
 * Creada por: Manuel Pereira
 * Fecha: 3-feb-2007
 * Modificada por: Daniel Prez y Alejandro Calvo
 * Fecha: 10-may-2007
 */

public class Casilla {
    public static final int TIPO_VACIA = 1;
    public static final int TIPO_MURO = 2;
    public static final int TIPO_PUNTO = 3;
    public static final int TIPO_FRUTA = 4;
    public static final int TIPO_GUARIDA = 5;

    /** Variable que representa el tipo de una casilla */
    private int tipo;
    
    /** Variable que representa si una casilla puede estar ocupada */    
    private boolean pisable;

    /**
     * Constructor de una casilla, que recibe su tipo
     * @param tipo El tipo de la casilla a crear
     */
    public Casilla(int tipo) {
        this.tipo = tipo;
        pisable = (tipo != TIPO_MURO && tipo != TIPO_GUARIDA);
    }

    /**
     * Comprueba si una casilla es pisable
     * @return true si es pisable, false si no lo es
     */
    public boolean esPisable() {
        return pisable;
    }

    /**
     * Comprueba si una casilla es un muro
     * @return true si es un muro, false si no lo es
     */
    public boolean esMuro() {
        return tipo == TIPO_MURO;
    }

    /**
     * Comprueba si una casilla est vaca
     * @return true si est vaca, false si no lo est
     */
    public boolean esVacia() {
        return tipo == TIPO_VACIA;
    }

    /**
     * Comprueba si una casilla es un punto
     * @return true si es un punto, false si no lo es
     */
    public boolean esPunto() {
        return tipo == TIPO_PUNTO;
    }

    /**
     * Comprueba si una casilla es una fruta
     * @return true si es un punto, false si no lo es
     */
    public boolean esFruta() {
        return tipo == TIPO_FRUTA;
    }
    
    /**
     * Comprueba si una casilla es una guarida
     * @return true si es un punto, false si no lo es
     */
    public boolean esGuarida() {
        return tipo == TIPO_GUARIDA;
    }
    
    /**
     * Selecciona si una casilla es o no pisable
     * @param pisable Indica si la casilla es o no pisable
     */
    public void setPisable(boolean pisable) {
        this.pisable = pisable;
    }
    
    /**
     * Selecciona el tipo de una casilla
     * @param tipo El nuevo tipo de la casilla
     */
    public void setTipo(int tipo) {
        this.tipo = tipo;
    }
    
    /**
     * Devuelve el tipo de la casilla
     * @return el tipo de casilla
     */
    public int getTipo() {
        return tipo;
    }
    
    
    /**
     * Devuelve la casilla representada en forma de cadena de texto
     * @return La casilla representada en forma de cadena de texto
     */
    public String toString() {
        switch(tipo) {
            case TIPO_VACIA:
                return " ";
            case TIPO_MURO:
                return "#";
            case TIPO_PUNTO:
                return ".";
            case TIPO_FRUTA:
            	return "$";
            case TIPO_GUARIDA:
            	return "O";
            default:
                return "";
        }
    }
}
