package practicas;

/**
 * Clase: Lista
 * Descripcin: Esta clase representa una lista de enemigos y los diferentes mtodos de ordenacin y bsqueda
 * Creada por: Roco Garca
 * Fecha: 3-mayo-2007
 */

public class Lista {

    /** Variable para la lista de enemigos */
    private Enemigo[] lista;

    /**
     * Constructor de una lista de enemigos. 
     */
    public Lista() {
        lista = null;
    }

    /**
     * Constructor de una lista de enemigos. 
     * @param listaE lista de enemigos
     */
    public Lista(Enemigo[] listaE) {
        lista = listaE;
    }

    /**
     * Devuelve la longitud de la lista
     * @return longitud de la lista
     */
    public int getLongitud(){
        return lista.length;
    }
    
    /**
     * Devuelve el enemigo situado en la posicion que se indica en el parametro
     * @param pos posicion del enemigo que se quiere obtener 
     * @return Enemigo que se quiere obtener
     */
    public Enemigo getEnemigo(int pos){
        return lista[pos];
    }
    
    public void mostrar(){
        System.out.println(" LISTA DE ENEMIGOS: ");
        for (int i=0; i<lista.length; i++){
            System.out.println(" ("+lista[i].getNombre()+","+lista[i].getDistancia()+")");
        }
        System.out.println();
    }

    /**
     * METODOS DE BSQUEDA ********************************************************
     */

    /**
     * Mtodo para buscar secuencialmente por el NOMBRE un enemigo en la lista de enemigos
     * @param nombre  nombre del enemigo a buscar 
     * @param desde   primer ndice de la lista
     * @param hasta   ltimo ndice de la lista
     * @return la posicin en la que se ha encontrado el elemento, o -1 si no se ha encontrado
     */
    public int busquedaSecuencial(String nombre, int desde, int hasta){

        for (int i=desde; i<=hasta; i++){
            if (lista[i].esEnemigo(nombre)) return i;
        }
        return -1;
    }

    /**
     * Mtodo para buscar secuencialmente por el NOMBRE un enemigo en la lista de enemigos
     * @param nombre nombre del enemigo a buscar
     * @return la posicin en la que se ha encontrado el elemento, o -1 si no se ha encontrado
     */
     public int busquedaSecuencial(String nombre){
	  int i;
	  i = busquedaSecuencial(nombre,0,lista.length);
        return i;
    }

    /**
     * Mtodo de bsqueda binaria por la DISTANCIA en una lista de enemigos
     * @param dist  distancia a la que se encuentra el enemigo a buscar
     * @param desde   primer ndice de la lista
     * @param hasta   ltimo ndice de la lista
     * @return la posicin en la que se ha encontrado el elemento, o -1 si no se ha encontrado
     */

    public int busquedaBinaria (double dist, int desde, int hasta){

        Enemigo aux = new Enemigo(0,0,0,"",0);
        for (;;){
            int medio = (desde + hasta)/2;
            if (lista[medio].igual(aux)) {
                return medio;
            }
            else if (desde>hasta) {
                return -1;
            }
            else if (lista[medio].menor(aux)) {
                desde = medio +1;
            }
            else {
                hasta = medio -1;
            }
        }
    }

    /**
     * Mtodo de bsqueda binaria por la DISTANCIA en una lista de enemigos
     */

    public int busquedaBinaria (double dist){
	int i;
	i = busquedaBinaria (dist,0,lista.length);
	return i;
    }
    
    /**
     * METODOS DE ORDENACIN *************************************************
     */


    /**
     * Mtodo de ordenacin burbuja
     * @param ultimo ltimo ndice de la lista a ordenar
     */
    public void burbuja (int ultimo){

        Enemigo aux;
        boolean cambio = true;

        for (int i=1; i<=ultimo && cambio; i++){
            cambio = false;
            for (int j=0; j<=ultimo-i; j++){
                if (!lista[j].menor(lista[j+1])){
                    cambio = true;
                    aux = lista [j+1];
                    lista [j+1] = lista [j];
                    lista [j] = aux;
                }
            }
        }
    }

    /**
     * Mtodo de ordenacin por insercin directa 
     * @param ultimo ltimo ndice de la lista a ordenar
     */
    public void insercionDirecta (int ultimo){

        Enemigo auxiliar;
        for (int i=1; i<=ultimo; i++){
            auxiliar = lista[i];
            int j=i-1;
            while (j>=0 && auxiliar.menor(lista[j])){
                lista [j+1]=lista[j];
                j--;
            }
            lista[j+1]=auxiliar;
        }
    }

    /**
     * Mtodo de ordenacin por seleccin Directa
     * @param ultimo ltimo ndice de la lista a ordenar
     */
    public void seleccionDirecta (int ultimo){

        Enemigo valorMenor;
        int posicionMenor;

        for (int i=0; i<=ultimo; i++) {
            posicionMenor=i;
            valorMenor=lista[i];
            for (int j=i+1; j<=ultimo; j++) {
                if (lista[j].menor(valorMenor)){
                    valorMenor=lista[j];
                    posicionMenor=j;
                }
            }
            lista[posicionMenor]=lista[i];
            lista[i]=valorMenor;
        }
    }

    /**
     * Mtodo de ordenacin montculo
     * @param ultimo ltimo ndice de la lista a ordenar
     */
    public void monticulo(int ultimo){
        Enemigo auxiliar;
        for (int i=(ultimo-1)/2; i>=0; i--) criba(i,ultimo);
        for (int i=ultimo; i>=1; i--){
            auxiliar=lista[0];
            lista[0]=lista[i];
            lista[i]=auxiliar;
            criba(0, i-1);
        }
    }

    /**
     * Mtodo de ayuda para la ordenacin del montculo
     * @param primero primer ndice de la lista a ordenar
     * @param ultimo  ltimo ndice de la lista a ordenar
     */
    private void criba (int primero, int ultimo){

        int indicePadre= primero;
        int indiceMenorHijo= 2*indicePadre+1;
        Enemigo auxiliar=lista[indicePadre];
        boolean salir= false;

        while (indiceMenorHijo<=ultimo && !salir) {
            if (indiceMenorHijo<ultimo && lista[indiceMenorHijo].menor(lista[indiceMenorHijo+1]))
                indiceMenorHijo++;
            if (auxiliar.mayor(lista[indiceMenorHijo]))
                salir= true;
            else {
                lista[indicePadre]=lista[indiceMenorHijo];
                indicePadre=indiceMenorHijo;
                indiceMenorHijo= indicePadre*2+1;
            }
        }
        lista[indicePadre]=auxiliar;
    }

    /**
     * Mtodo de ordenacin rpido
     * @param primero primer ndice de la lista a ordenar
     * @param ultimo  ltimo ndice de la lista a ordenar
     */
    public void metodoRapido (int primero, int ultimo){
        int i=primero, j=ultimo;
        Enemigo pivote = lista[(primero+ultimo)/2];
        Enemigo auxiliar;

        do {
            while (lista[i].menor(pivote)) {
                i++;
            }
            while (lista[j].mayor(pivote)) {
                j--;
            }
            if (i<=j){
                auxiliar=lista[j];
                lista[j]=lista[i];
                lista[i]=auxiliar;
                i++;
                j--;
            }
        } while (i<=j);
        if (primero<j) {
            metodoRapido(primero, j);
        }
        if (i<ultimo) {
            metodoRapido(i, ultimo);
        }
    }





}
