package practicas;

/**
 * Clase: Protagonista
 * Descripcin: Esta clase representa al protagonista en el juego del comecocos. La clase
 * hereda de la clase Individuo que contiene los mtodos comunes al protagonista y los enemigos
 * Creada por: Manuel Pereira
 * Fecha: 13-feb-2007
 */

public class Protagonista extends Individuo {

    /**
     * Constructor que recibe toda la informacin sobre el protagonista
     * @param posicionX Posicin X inicial del protagonista
     * @param posicionY Posicin Y inicial del protagonista
     * @param direccion Direccin inicial del protagonista (las constantes para
     *        las distintas direcciones estn definidas en la clase Individuo)
     */
    public Protagonista(int posicionX, int posicionY, int direccion) {
        super(posicionX, posicionY, direccion);
    }

    /**
     * Mtodo que indica al protagonista que se mueva. Para ello se le pasan como
     * parmetro las casillas del tablero
     * @param casillas Las casillas del tablero
     * @param teclaPulsada Tecla pulsada por el usuario final
     */
    public void muevete(Casilla[][] casillas, String teclaPulsada) {

        // Modifica la direccin si se ha pulsado una de las teclas de cambio de direccin
        if(teclaPulsada.equals("W") || teclaPulsada.equals("w")) {
            direccion = DIRECCION_ARRIBA;
        }
        else if(teclaPulsada.equals("S") || teclaPulsada.equals("s")) {
            direccion = DIRECCION_ABAJO;
        }
        else if(teclaPulsada.equals("A") || teclaPulsada.equals("a")) {
            direccion = DIRECCION_IZQUIERDA;
        }
        else if(teclaPulsada.equals("D") || teclaPulsada.equals("d")) {
            direccion = DIRECCION_DERECHA;
        }
        // Si se ha pulsado la tecla Q, sale de la partida
        else if(teclaPulsada.equals("Q") || teclaPulsada.equals("q")) {
        	System.out.println("Salimos del juego");
            System.exit(-1);
        }

        // Invoca al mtodo de movimiento de la clase padre
        super.muevete(casillas);
    }

    /**
     * Cambia la direccion del protagonista segn la tecla pulsada
     * @param teclaPulsada Tecla pulsada por el usuario final
     */
    public void cambiaDireccion(String teclaPulsada) {

        // Modifica la direccin si se ha pulsado una de las teclas de cambio de direccin
        if(teclaPulsada.equals("W") || teclaPulsada.equals("w")) {
            direccion = DIRECCION_ARRIBA;
        }
        else if(teclaPulsada.equals("S") || teclaPulsada.equals("s")) {
            direccion = DIRECCION_ABAJO;
        }
        else if(teclaPulsada.equals("A") || teclaPulsada.equals("a")) {
            direccion = DIRECCION_IZQUIERDA;
        }
        else if(teclaPulsada.equals("D") || teclaPulsada.equals("d")) {
            direccion = DIRECCION_DERECHA;
        }
        // Si se ha pulsado la tecla Q, sale de la partida
        else if(teclaPulsada.equals("Q") || teclaPulsada.equals("q")) {
            System.exit(-1);
        }
    }

}
