package practicas;

import java.util.Random;

/**
 * Clase: Tablero
 * Descripcin: Esta clase representa un tablero para el juego comecocos
 * Creada por: Manuel Pereira
 * Fecha: 3-feb-2007
 * Modificada por: Daniel Prez y Alejandro Calvo
 * Fecha: 10-may-2007
 */

public class Tablero {

    /** Casillas del tablero */
    private Casilla[][] casillas;

    /** Lista de enemigos */
    private Lista enemigos;

    /** Protagonista */
    private Protagonista prota;

    /** Ancho del tablero */
    private int ancho;

    /** Alto del tablero */
    private int alto;

    /** Vidas restantes **/
    private int vidas;
    
    /** Puntos acumulados */
    private int puntos;

    /** Puntos comidos **/
    private int puntosComidos;
    
    /** Puntos iniciales **/
    private int puntosIniciales;
    
    /** Indica si la fruta ha sido ya generada **/
    private boolean frutaGenerada;
    
    /** Constante para el incremento de puntos cada vez que se come uno */
    private static final int INCREMENTO_PUNTOS = 10;
    
    /** Constante para el nemro de vidas iniciales */
    private static final int VIDAS_INICIALES = 15;

    /**
     * Constructor del tablero dados su ancho, su alto, sus casillas, los enemigos y el protagonista
     * @param ancho Ancho del tablero
     * @param alto Alto del tablero
     * @param casillas Casillas de juego
     * @param enemigos Enemigos
     * @param prota Protagonista
     */
    public Tablero(Lista enemigos, Protagonista prota) {

    	// Asigna las variables del tablero
        this.enemigos = enemigos;
        this.prota = prota;
        
        // El nmero de vidas vendr indicado por VIDAS_INICIALES
        this.vidas = VIDAS_INICIALES;
        
        // Inicialmente la fruta no ha sido generada
        this.frutaGenerada = false;
        
        // Inicialmente los puntos son cero
        this.puntos = 0;
    }
    
    /**
     * Construye el tablero inicial
     */
    public void crearTableroInicial(){
    	/* Tablero */
        /*
        /*    ############################
        /*    #............##............#
        /*    #.####.#####.##.#####.####.#
        /*    #*####.#####.##.#####.####*#
        /*    #.####.#####.##.#####.####.#
        /*    #..........................#
        /*    #.####.##.########.##.####.#
        /*	  #.####.##.########.##.####.#
        /*    #......##....##....##......#
        /*	  ######.####  ##  ####.######
        /*    ######.####  ##  ####.######
        /*    ######.##          ##.######
        /*    ######.## ##OOOO## ##.######
        /*    ######.## ######## ##.######
        /*    ######.   ########   .######
        /*    ######.## ######## ##.######
        /*    ######.## ######## ##.######
        /*    ######.##          ##.######
        /*    ######.## ######## ##.######
        /*    ######.## ######## ##.######
        /*    #............##............#
   	    /*    #.####.#####.##.#####.####.#
        /*    #.####.#####.##.#####.####.#
        /*    #*..##.......@........##..*#
        /*    ###.##.##.########.##.##.###
        /*    ###.##.##.########.##.##.###
        /*    #......##....##....##......#
        /*    #.##########.##.##########.#
        /*    #.##########.##.##########.#
        /*    #..........................#
        /*    ############################
         */

    	int muro = Casilla.TIPO_MURO;
        int pnto = Casilla.TIPO_PUNTO;
        int grda = Casilla.TIPO_GUARIDA;
        int vcia = Casilla.TIPO_VACIA;
        int[][] tmp = new int[][] {
	      	{muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro},
	      	{muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro,muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
	      	{muro,pnto,muro,muro,muro,muro,pnto,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,pnto,muro,muro,muro,muro,pnto,muro},
	      	{muro,pnto,muro,muro,muro,muro,pnto,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,pnto,muro,muro,muro,muro,pnto,muro},
	      	{muro,pnto,muro,muro,muro,muro,pnto,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,pnto,muro,muro,muro,muro,pnto,muro},
	      	{muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
	      	{muro,pnto,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,pnto,muro},
	      	{muro,pnto,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,pnto,muro},
	      	{muro,pnto,pnto,pnto,pnto,pnto,pnto,muro,muro,pnto,pnto,pnto,pnto,muro,muro,pnto,pnto,pnto,pnto,muro,muro,pnto,pnto,pnto,pnto,pnto,pnto,muro},
	      	{muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,muro,muro,vcia,muro,muro,vcia,muro,muro,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro},
	      	{muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,muro,muro,vcia,muro,muro,vcia,muro,muro,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro},
	      	{muro,muro,muro,muro,muro,muro,pnto,muro,muro,vcia,vcia,vcia,vcia,vcia,vcia,vcia,vcia,vcia,vcia,muro,muro,pnto,muro,muro,muro,muro,muro,muro},
	      	{muro,muro,muro,muro,muro,muro,pnto,muro,muro,vcia,muro,muro,grda,grda,grda,grda,muro,muro,vcia,muro,muro,pnto,muro,muro,muro,muro,muro,muro},
	      	{muro,muro,muro,muro,muro,muro,pnto,vcia,vcia,vcia,muro,muro,muro,muro,muro,muro,muro,muro,vcia,vcia,vcia,pnto,muro,muro,muro,muro,muro,muro},
	      	{muro,muro,muro,muro,muro,muro,pnto,muro,muro,vcia,muro,muro,muro,muro,muro,muro,muro,muro,vcia,muro,muro,pnto,muro,muro,muro,muro,muro,muro},
	      	{muro,muro,muro,muro,muro,muro,pnto,muro,muro,vcia,vcia,vcia,vcia,vcia,vcia,vcia,vcia,vcia,vcia,muro,muro,pnto,muro,muro,muro,muro,muro,muro},
	      	{muro,muro,muro,muro,muro,muro,pnto,muro,muro,vcia,muro,muro,muro,muro,muro,muro,muro,muro,vcia,muro,muro,pnto,muro,muro,muro,muro,muro,muro},
	      	{muro,muro,muro,muro,muro,muro,pnto,muro,muro,vcia,muro,muro,muro,muro,muro,muro,muro,muro,vcia,muro,muro,pnto,muro,muro,muro,muro,muro,muro},
	      	{muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro,muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
	      	{muro,pnto,muro,muro,muro,muro,pnto,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,pnto,muro,muro,muro,muro,pnto,muro},
	      	{muro,pnto,muro,muro,muro,muro,pnto,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,pnto,muro,muro,muro,muro,pnto,muro},
	      	{muro,pnto,pnto,pnto,muro,muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro,muro,pnto,pnto,pnto,muro},
	      	{muro,muro,muro,pnto,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,pnto,muro,muro,muro},
	      	{muro,muro,muro,pnto,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,pnto,muro,muro,muro},
	      	{muro,pnto,pnto,pnto,pnto,pnto,pnto,muro,muro,pnto,pnto,pnto,pnto,muro,muro,pnto,pnto,pnto,pnto,muro,muro,pnto,pnto,pnto,pnto,pnto,pnto,muro},
	      	{muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro},
	      	{muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro},
	      	{muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
	      	{muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro}};

        this.alto = tmp.length;
        this.ancho = tmp[0].length;
       	int ptos = 0;

        // Crea las casillas y calcula puntos iniciales en el tablero
        this.casillas = new Casilla[this.alto][this.ancho];
        for(int i=0; i<this.alto; i++) {
        	casillas[i] = new Casilla[this.ancho];
            for(int j=0; j<this.ancho; j++) {
                casillas[i][j] = new Casilla(tmp[i][j]);
                if(casillas[i][j].esPunto()) {
                	ptos++;
                }
            }
        }
        this.puntosIniciales=ptos;
    }

    /**
     * Construye el tablero inicial
     */
    public void crearTableroSecundario(){
    	/* Tablero */
        /*
        /*    ############################
        /*    #..........................#
        /*    #.###.###.########.###.###.#
        /*    #*###.###.########.###.###*#
        /*    #.###.###.########.###.###.#
        /*    #..........................#
        /*    #.###.###.########.###.###.#
        /*    #.###.###.########.###.###.#
        /*    #.###.###.########.###.###.#
        /*    #..........................#
        /*    #.####.##.########.##.####.#
        /*    #.####.##.########.##.####.#
        /*    #.####.##.##OOOO##.##.######
        /*    #.####.##.##    ##.##.####.#
        /*    #..........................#
        /*    #.###.###.########.###.###.#
        /*    #.###.###.########.###.###.#
        /*    #.###.###.########.###.###.#
        /*    #..........................#
        /*    #.###.###.########.###.###.#
        /*    #.###.###.########.###.###.#
        /*    #............@.............#
        /*    #.###.###.########.###.###.#
        /*    #..........................#
        /*    #.###.###.########.###.###.#
        /*    #.###.###.########.###.###.#
        /*    #..........................#
        /*    #*###.###.########.###.###*#
        /*    #.###.###.########.###.###.#
        /*    #..........................#
        /*    ############################
         */

    	int muro = Casilla.TIPO_MURO;
        int pnto = Casilla.TIPO_PUNTO;
        int grda = Casilla.TIPO_GUARIDA;
        int vcia = Casilla.TIPO_VACIA;
        int[][] tmp = new int[][] {
        		{muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro},
                {muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
                {muro,pnto,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,grda,grda,grda,grda,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,muro,pnto,muro,muro,pnto,muro,muro,vcia,vcia,vcia,vcia,muro,muro,pnto,muro,muro,pnto,muro,muro,muro,muro,pnto,muro},
                {muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,muro,muro,muro,muro,muro,pnto,muro,muro,muro,pnto,muro,muro,muro,pnto,muro},
                {muro,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,pnto,muro},
                {muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro,muro}};
       
        this.alto = tmp.length;
        this.ancho = tmp[0].length;
       	int ptos = 0;

        // Crea las casillas y calcula puntos iniciales en el tablero
        this.casillas = new Casilla[this.alto][this.ancho];
        for(int i=0; i<this.alto; i++) {
        	casillas[i] = new Casilla[this.ancho];
            for(int j=0; j<this.ancho; j++) {
                casillas[i][j] = new Casilla(tmp[i][j]);
                if(casillas[i][j].esPunto()) {
                	ptos++;
                }
            }
        }
        this.puntosIniciales=ptos;
    }

    /**
     * Realiza el siguiente movimiento del tablero, moviendo tanto a los
     * enemigos como al protagonista. Devuelve false si se ha perdido porque algn
     * enemigo ha alcanzado al protagonista
     * @param teclaPulsada Tecla pulsada por el usuario
     */
    public void siguienteMovimiento(String teclaPulsada) {

    	// Se almacena en estas variables la posicin inicial del protagonista (antes de mover)
        int protaX = prota.getPosicionX();
        int protaY = prota.getPosicionY();
    	
//      Suma puntos al marcador si es necesario
        if(casillas[prota.getPosicionY()][prota.getPosicionX()].esPunto()) {
            puntos += INCREMENTO_PUNTOS;
            puntosComidos++;
            casillas[prota.getPosicionY()][prota.getPosicionX()].setTipo(Casilla.TIPO_VACIA);
        }
        
    	// Primero mueve al protagonista
        prota.muevete(casillas, teclaPulsada);

        // Suma puntos al marcador si es necesario
        if(casillas[prota.getPosicionY()][prota.getPosicionX()].esPunto()) {
            puntos += INCREMENTO_PUNTOS;
            puntosComidos++;
            casillas[prota.getPosicionY()][prota.getPosicionX()].setTipo(Casilla.TIPO_VACIA);
        }

        // Suma puntosx10 al marcador al comer la casilla Fruta 
        if(casillas[prota.getPosicionY()][prota.getPosicionX()].esFruta()) {
            puntos += INCREMENTO_PUNTOS*10;
            casillas[prota.getPosicionY()][prota.getPosicionX()].setTipo(Casilla.TIPO_VACIA);
        }
        
        // Se comprueba si se ha perdido una vida
        // El protagonista ha movido a una casilla ocupada por un enemigo
        if(vidaPerdida()) {
        	// Si las vidas han sido reducidas a cero el juego finaliza
            if(juegoPerdido()) {
                System.out.println("Has perdido la partida!");
                System.exit(-1);
            }
            // Si todava quedan vidas, el juego se reinicia
            // Protagonista y enemigos comienzan desde la posicin de partida
            else{
            	System.out.println("Has perdido una vida!");
            	nuevaVida();
            }
        }

        // Se comprueba si se ha de generar la fruta
        generarFruta();
        
        // Se mueven a los enemigos
        for(int i=0; i<enemigos.getLongitud(); i++) {
            enemigos.getEnemigo(i).muevete(casillas,enemigos,protaX,protaY);
        }
        
        // Se comprueba si se ha perdido una vida
        // Uno de los enemigos ha alcanzado al protagonista
        if(vidaPerdida()) {
        	// Si las vidas han sido reducidas a cero el juego finaliza
            if(juegoPerdido()) {
                System.out.println("Has perdido la partida!");
                System.exit(-1);
            }
            // Si todava quedan vidas, el juego se reinicia
            // Protagonista y enemigos comienzan desde la posicin de partida
            else{
            	System.out.println("Has perdido una vida!");
            	nuevaVida();
            }
        }
        //Cada turno utilizamos un metodo de ordenacion seleccionado aleatoriamente
        Random aleatorio = new Random(System.currentTimeMillis());
        int metodo=aleatorio.nextInt(5);
        switch(metodo){
	        case 0:{
	            enemigos.burbuja(enemigos.getLongitud()-1);
	        	break;
	        }
	        case 1:{
	        	enemigos.insercionDirecta(enemigos.getLongitud()-1);
	        	break;
	        }
	        case 2:{
	        	enemigos.seleccionDirecta(enemigos.getLongitud()-1);
	        	break;
	        }
	        case 3:{
	        	enemigos.monticulo(enemigos.getLongitud()-1);
	        	break;
	        }
	        case 4:{
	        	enemigos.metodoRapido(0,enemigos.getLongitud()-1);
	        	break;
	        }
        }
        
    }
    
    /**
     * Realiza el siguiente movimiento del tablero, moviendo tanto a los
     * enemigos como al protagonista. Devuelve false si se ha perdido porque algn
     * enemigo ha alcanzado al protagonista
     * @param teclaPulsada Tecla pulsada por el usuario
     */
    public void siguienteMovimiento() {

    	// Se almacena en estas variables la posicin inicial del protagonista (antes de mover)
        int protaX = prota.getPosicionX();
        int protaY = prota.getPosicionY();
    	
//      Suma puntos al marcador si es necesario
        if(casillas[prota.getPosicionY()][prota.getPosicionX()].esPunto()) {
            puntos += INCREMENTO_PUNTOS;
            puntosComidos++;
            casillas[prota.getPosicionY()][prota.getPosicionX()].setTipo(Casilla.TIPO_VACIA);
        }
        
    	// Primero mueve al protagonista
        prota.muevete(casillas);

        // Suma puntos al marcador si es necesario
        if(casillas[prota.getPosicionY()][prota.getPosicionX()].esPunto()) {
            puntos += INCREMENTO_PUNTOS;
            puntosComidos++;
            casillas[prota.getPosicionY()][prota.getPosicionX()].setTipo(Casilla.TIPO_VACIA);
        }

        // Suma puntosx10 al marcador al comer la casilla Fruta 
        if(casillas[prota.getPosicionY()][prota.getPosicionX()].esFruta()) {
            puntos += INCREMENTO_PUNTOS*10;
            casillas[prota.getPosicionY()][prota.getPosicionX()].setTipo(Casilla.TIPO_VACIA);
        }
        
        // Se comprueba si se ha perdido una vida
        // El protagonista ha movido a una casilla ocupada por un enemigo
        if(vidaPerdida()) {
        	// Si las vidas han sido reducidas a cero el juego finaliza
            if(juegoPerdido()) {
                System.out.println("Has perdido la partida!");
                System.exit(-1);
            }
            // Si todava quedan vidas, el juego se reinicia
            // Protagonista y enemigos comienzan desde la posicin de partida
            else{
            	System.out.println("Has perdido una vida!");
            	nuevaVida();
            }
        }

        // Se comprueba si se ha de generar la fruta
        generarFruta();
        
        // Se mueven a los enemigos
        for(int i=0; i<enemigos.getLongitud(); i++) {
            enemigos.getEnemigo(i).muevete(casillas,enemigos,protaX,protaY);
        }
        
        // Se comprueba si se ha perdido una vida
        // Uno de los enemigos ha alcanzado al protagonista
        if(vidaPerdida()) {
        	// Si las vidas han sido reducidas a cero el juego finaliza
            if(juegoPerdido()) {
                System.out.println("Has perdido la partida!");
                System.exit(-1);
            }
            // Si todava quedan vidas, el juego se reinicia
            // Protagonista y enemigos comienzan desde la posicin de partida
            else{
            	System.out.println("Has perdido una vida!");
            	nuevaVida();
            }
        }
        //Cada turno utilizamos un metodo de ordenacion seleccionado aleatoriamente
        Random aleatorio = new Random(System.currentTimeMillis());
        int metodo=aleatorio.nextInt(5);
        switch(metodo){
	        case 0:{
	            enemigos.burbuja(enemigos.getLongitud()-1);
	        	break;
	        }
	        case 1:{
	        	enemigos.insercionDirecta(enemigos.getLongitud()-1);
	        	break;
	        }
	        case 2:{
	        	enemigos.seleccionDirecta(enemigos.getLongitud()-1);
	        	break;
	        }
	        case 3:{
	        	enemigos.monticulo(enemigos.getLongitud()-1);
	        	break;
	        }
	        case 4:{
	        	enemigos.metodoRapido(0,enemigos.getLongitud()-1);
	        	break;
	        }
        }
        
    }
    
    /**
     * Comprueba si se han comido la mitad de los puntos totales del tablero.
     * En caso de ser as, genera una casilla de tipo Fruta en un lugar aleatorio del tablero.
     */
    public void generarFruta() {
    	// Se comprueba si la fruta ha sido ya generada y si se han comido la mitad de los puntos
    	if (!frutaGenerada&&(puntosComidos >= (puntosIniciales/2))) {
    		// Se generan posiciones aleatorias dentro del tablero
    		do{
                Random rnd1 = new Random();
            	Random rnd2 = new Random();
               	int posicionX = 1+rnd1.nextInt(ancho-1);
               	int posicionY = 1+rnd2.nextInt(alto-1);
            	if (casillas[posicionY][posicionX].esVacia()){
                	casillas[posicionY][posicionX].setTipo(Casilla.TIPO_FRUTA);
                	frutaGenerada = true;
                	}
            	// Hasta que se selecciona una casilla vaca
            	}while(!frutaGenerada);
    	}
    }

    /**
     * Comprueba si se ha perdido una vida, viendo si algn enemigo est en la misma
     * casilla que el protagonista
     * @return true si se ha perdido, false si no
     */
    public boolean vidaPerdida() {
        // Comprueba si algn enemigo ha alcanzado al protagonista
        for(int i=0; i<enemigos.getLongitud(); i++) {
            if(enemigos.getEnemigo(i).coincide(prota)) {
            	// Se resta una vida
            	vidas--;
                return true;
            }
        }
        return false;
    }

    
    /**
     * Comprueba si se ha perdido el juego, viendo las vidas restantes
     * @return true si se ha perdido, false si no
     */
    public boolean juegoPerdido() {
        // Comprueba si el nmero de vidas resantes es cero
        return vidas==0;
    }
    
    /**
     * Comprueba si se ha ganado el juego, viendo que no quede ninguna casilla con puntos
     * @return true si se ha ganado, false si no
     */
    public boolean juegoGanado() {
    	// Se comprueba que no queda ningn casilla tipo punto sobre el tablero
        for(int i=0; i<alto; i++) {
            for(int j=0; j<ancho; j++) {
                if(casillas[i][j].esPunto()) {
                    return false;
                }
            }
        }
        return true;
    }
    
    /**
     * Devuelve al protagonista y a los enemigos a sus posiciones iniciales
     */
    public void nuevaVida() {
    	// Se recorre la lista de enemigos y se va situando a todos en su situacin inicial
        for(int i=0; i<enemigos.getLongitud(); i++) {
            enemigos.getEnemigo(i).situarInicio();
        }
        // Se sita al protagonista en su situacin inicial
        prota.situarInicio();
    }
    
    /**
     * Devuelve la representacin del tablero en forma de cadena de texto
     * @return La representacin del tablero en forma de cadena de texto
     */
    public String mostrarTablero() {
        String ret = "";
        
        // Se imprime la lista de enemigos ordenada por distancia
        ret += "****************************" + "\n";
        ret += " ENEMIGOS " + "\n";
        ret += "****************************" + "\n";
        for(int k=0; k<enemigos.getLongitud(); k++) {
           	ret += " " + enemigos.getEnemigo(k).getNombre();
           	ret += " (" + enemigos.getEnemigo(k).getColor() + ") ";
           	ret += " " + enemigos.getEnemigo(k).getDistancia() + "\n";
        }
        ret += "****************************" + "\n";
        
        // Se recorre todo el tablero
        for(int i=0; i<alto; i++) {
            for(int j=0; j<ancho; j++) {
                // Si est el protagonista lo imprime
                if(prota.coincide(j, i)) {
                    ret += "@";
                }
                else {
                    boolean tmp = true;
                    // Si hay algn enemigo en esa casilla, lo imprime
                    for(int k=0; k<enemigos.getLongitud(); k++) {
                        if(enemigos.getEnemigo(k).coincide(j, i)) {
                            ret += enemigos.getEnemigo(k).getInicial();
                            tmp = false;
                        }
                    }
                    // Si no, imprime la casilla
                    if(tmp) {
                        ret += casillas[i][j].toString();
                    }
                }
            }
            ret += "\n";
        }
        ret += "VIDAS: " + vidas + "\n";
        ret += "PUNTOS: " + puntos + "\n";
        return ret;
    }

    /**
     * Devuelve las casillas del juego
     * @return Las casillas del juego
     */
    public Casilla[][] getCasillas() {
        return casillas;
    }

    /**
     * Devuelve los enemigos del juego
     * @return Los enemigos del juego
     */
    public Lista getEnemigos() {
        return enemigos;
    }

    /**
     * Devuelve el protagonista del juego
     * @return El protagonista del juego
     */
    public Protagonista getProta() {
        return prota;
    }

    /**
     * Devuelve el ancho del tablero
     * @return El ancho del tablero
     */
    public int getAncho() {
        return ancho;
    }

    /**
     * Devuelve el alto del tablero
     * @return El alto del tablero
     */
    public int getAlto() {
        return alto;
    }

    /**
     * Devuelve los puntos actuales del juego
     * @return Los puntos actuales del juego
     */
    public int getPuntos() {
        return puntos;
    }
    
    /**
     * Establece los puntos del juego
     * @param ptos Los puntos actuales del juego
     */
    public void setPuntos(int ptos) {
        puntos=ptos;
    }
}
